@@ -1,4 +1,4 @@
-# $Id: Test.pm,v 1.3 2008-04-10 00:18:13 mike Exp $
+# $Id: Test.pm,v 1.4 2008-04-11 12:03:30 mike Exp $
 
 package Keystone::Resolver::Test;
 
@@ -108,8 +108,8 @@ sub _do_test {
 
     my $cgi = new CGI($params);
     my $resolver = new Keystone::Resolver();
-    my $openURL = Keystone::Resolver::OpenURL->newFromCGI($cgi, undef,
-	{ baseURL => "http://example.com/resolve", %$optsref }, $resolver);
+    my $openURL = Keystone::Resolver::OpenURL->newFromCGI($resolver, $cgi, undef,
+	{ baseURL => "http://example.com/resolve", %$optsref });
     my($__UNUSED_type, $result) = $openURL->resolve();
 
     if ($result !~ /\n$/s) {
